package com.shixing.sxedit;

import com.shixing.sxedit.types.SXPreviewScale;

/**
 * 剪辑参数
 *
 * */

public class SXEditOptions {

    /**
     * 剪辑宽度，默认720
     * */
    private int width = 720;

    /**
     * 剪辑高度，默认1280
     * */
    private int height = 1280;

    /**
     * 帧速率，默认20
     * */
    private int fps = 20;

    /**
     * 所需验证的 license
     * */
    private String license = "";

    /**
     * 整体渲染缩放倍数
     */
    private SXPreviewScale scale = SXPreviewScale.Scale100;

    /**
     * 文字默认字体路径
     * */
    private String fontFile = null;

    /**
     * 是否开启素材预加载，默认不开启
     * */
    private boolean enableSourceManager = false;

    /**
     * 素材预加载的预留空间大小, 默认300M, 单位 M
     * */
    private long cacheSize = 300;

    private boolean changed = false;



    /**
     * 构造函数
     *
     * @param license 授权证书
     * @param width 宽度 px
     * @param height 高度 px
     * @param fps 帧率
     *
     * */
    public SXEditOptions(String license, int width, int height, int fps) {
        this.license = license;
        this.width = width;
        this.height = height;
        this.fps = fps;
        this.changed = true;
    }

    /**
     * 释放创建的 EditOptions 对象, 必须调用
     * */
    public void destroy(){
        if (mNativeOptions > 0) {
            nDeleteEditOptions(mNativeOptions);
            mNativeOptions = 0;
        }
    }

    /**
     * 获取宽度, px
     *
     * @return 宽度, px
     * */
    public int getWidth() {
        return width;
    }

    /**
     * 设置宽度
     *
     * @param width 宽度 px
     * */
    public void setWidth(int width) {
        this.width = width;
        this.changed = true;
    }

    /**
     * 获取高度, px
     *
     * @return 高度, px
     * */
    public int getHeight() {
        return height;
    }

    /**
     * 设置高度
     *
     * @param height 高度 px
     * */
    public void setHeight(int height) {
        this.height = height;
        this.changed = true;
    }

    /**
     * 获取帧率
     *
     * @return int, fps
     * */
    public int getFps() {
        return fps;
    }

    /**
     * 设置帧率
     *
     * @param fps 帧率, 默认为 20
     * */
    public void setFps(int fps) {
        this.fps = fps;
        this.changed = true;
    }

    /**
     * 获取 license
     *
     * @return license
     * */
    public String getLicense() {
        return license;
    }

    /**
     * 设置 license
     *
     * @param license 证书
     * */
    public void setLicense(String license) {
        this.license = license;
        this.changed = true;
    }

    /**
     * 获取缩放比例
     *
     * @return {@link SXPreviewScale}
     * */
    public SXPreviewScale getScale() {
        return scale;
    }

    /**
     * 设置缩放比例
     *
     * @param scale 缩放比例, 默认 100
     * */
    public void setScale(SXPreviewScale scale) {
        this.scale = scale;
        this.changed = true;
    }

    /**
     * 获取默认字体路径
     *
     * @return 字体路径
     * */
    public String getFontFile() {
        return fontFile;
    }
    /**
     * 设置默认字体路径
     *
     * @param fontFile 字体路径
     * */
    public void setFontFile(String fontFile) {
        this.fontFile = fontFile;
        this.changed = true;
    }

    /**
     * 获取素材缓存是否开启
     *
     * @return boolean, 是否开启
     * */
    public boolean isEnableSourceManager() {
        return enableSourceManager;
    }

    /**
     * 设置是否启用素材管理
     * <p>
     *     1. 服务端不建议开启
     * </p>
     * @param enableSourceManager, 是否启用
     * */
    public void setEnableSourceManager(boolean enableSourceManager) {
        this.enableSourceManager = enableSourceManager;
        this.changed = true;
    }

    /**
     * 获取缓存大小, 单位: 字节
     *
     * @return long, size
     * */
    public long getCacheSize() {
        return cacheSize;
    }

    /**
     * 设置缓存大小, 默认 300M
     *
     * @param cacheSize 大小, 单位: MB
     * */
    public void setCacheSize(long cacheSize) {
        this.cacheSize = cacheSize;
        this.changed = true;
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        destroy();
    }

    public long getNativeOptions() {
        if (mNativeOptions == 0) {
            mNativeOptions = nCreateEditOptions(license, width, height, fps, scale.ordinal(), fontFile, enableSourceManager, cacheSize);
        } else if (mNativeOptions > 0 && changed) {
            nDeleteEditOptions(mNativeOptions);
            mNativeOptions = nCreateEditOptions(license, width, height, fps, scale.ordinal(), fontFile, enableSourceManager, cacheSize);
        }
        return mNativeOptions;
    }

    private long mNativeOptions;

    private static native long nCreateEditOptions(String license, int width, int height, int fps, int ordinal, String fontFile, boolean enableSourceManager, long cacheSize);

    private static native void nDeleteEditOptions(long mNativeOptions);
}
